 /*
  * entry.c
  * Copyright (C) Aitor Cuadrado Zubizarreta <aitor_czr@gnuinos.org>
  * 
  * simple-netaid is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
  * Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
  * 
  * simple-netaid is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  * See the GNU General Public License for more details.
  * 
  * You should have received a copy of the GNU General Public License along
  * with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * See the COPYING file.
  */
   
 
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <unistd.h>
#include <stdbool.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <limits.h>         /*  PATH_MAX  */
#include <unistd.h>
#include <sys/un.h>           /*  UNIX socket  */
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <assert.h>
#include <stdarg.h>

#include "entry.h"
#include "util.h"
#include "def.h"

#define MAX_SIZE 1024

// https://jdebp.uk/FGA/predefined-macros-processor.html 
#if defined(__x86_64__)
const char lib_interp[] __attribute__((section(".interp"))) = "/lib64/ld-linux-x86-64.so.2";
#elif defined(__i386__)
const char lib_interp[] __attribute__((section(".interp"))) = "/lib/ld-linux.so.2"; 
#endif

void emit_sigusr(int value, const char *fullpath)
{
   /* System V IPC key */
   key_t key;        
   int *shm_PTR;   
   /* System V shared memory segment associated with the value of the IPC key */
   int shm_ID;  
   char cwd_buf[ PATH_MAX + 1 ];
   
   // Get the current directory
   memset( cwd_buf, 0, PATH_MAX + 1 );   
   char* tmp = getcwd( cwd_buf, PATH_MAX );
   if(!tmp) {
      log_error("Current working directory exceeds %u bytes\n", PATH_MAX);
      exit(EXIT_FAILURE);
   }
  
   // Send the argument (an integer) through the shared memory segment
   key = ftok(cwd_buf, 's');    
   shm_ID = shmget(key, sizeof(int), IPC_CREAT | 0666);
   shm_PTR = (int *) shmat(shm_ID, NULL, 0);
   *shm_PTR = value;
 
   // Run the shared library
   system(fullpath);
   shmdt(shm_PTR);  
   shmctl(shm_ID, IPC_RMID, NULL);
}

/**
 * \brief Variadic function
 */
void send_arguments(int option, int count, ...) 
{               
   int dfClient = -1, dfData = -1;
   struct sockaddr_un dirUNIXClient;
   int sz, rc, res = 0;
      
   va_list items;
   
   va_start(items, count);  /* Requires the last fixed parameter (to get the address) */
   
   signal(SIGCHLD, SIG_IGN);    /*  avoid zombies */
   
   /*
    * In case the program exited inadvertently on the last run,
    * remove the socket.
    */
   unlink(SOCKET_NAME);
   
   
   dfClient = socket(AF_UNIX, SOCK_STREAM, PROTOCOL);
   if(dfClient == -1) {
      perror("socket");
      exit(EXIT_FAILURE);
   }
    
   /*
    * For portability clear the whole structure, since some
    * implementations have additional (nonstandard) fields in
    * the structure.
    */
   memset(&dirUNIXClient, 0, sizeof(struct sockaddr_un)); 
   
   dirUNIXClient.sun_family = AF_UNIX;
   sz = sizeof(dirUNIXClient.sun_path) - 1;
   strncpy_t(dirUNIXClient.sun_path, sz, SOCKET_NAME, sz);
   rc = bind(dfClient, (const struct sockaddr*) &dirUNIXClient, sizeof(struct sockaddr_un));
   if (rc == -1) {
      perror("bind");
      exit(EXIT_FAILURE);
   }
    
   res = listen(dfClient, 5);
   if(res == -1) {
      perror("listen");
      exit(EXIT_FAILURE);
   }
        
   dfData = accept(dfClient, NULL, NULL);
   if (dfData == -1) {
      perror("accept");
      exit(EXIT_FAILURE);
   }
                                 
  //if(!fork()) {
   
      char str[BUFFER_SIZE];

      sprintf(str, "%d", option);   
      rc = write(dfData, str, BUFFER_SIZE);
      if (rc == -1) {
         perror("write");
      }

      sprintf(str, "%d", count);   
      rc = write(dfData, str, BUFFER_SIZE);
      if (rc == -1) {
         perror("write");
      }
    
      while(count--) {
        rc = write(dfData, va_arg(items, const char*), BUFFER_SIZE);
         if (rc == -1) {
            perror("write");
         }
      }
   //}
   
   
   va_end(items);
   
   close(dfData);
   close(dfClient);
}

void get_daemon_pid(struct sbuf *s)
{    
   FILE *fp = NULL;
   char *token = NULL;
   struct sbuf cmd;
  
   sbuf_init(&cmd);
   sbuf_addstr(&cmd, "ps -A | grep \"snetaid$\"");
   fp=popen(cmd.buf, "r");
   if(!fp) {
      printf("Sorry, there was an error opening the pipe\n");
      free(cmd.buf);
      exit(EXIT_FAILURE);
   }
   sbuf_free(&cmd);
  
   int count=0;
   char line[MAX_SIZE]={0};
   while (fgets(line, MAX_SIZE, fp)) {
      line[strcspn(line, "\n")] = 0;
      const char delim[2] = " ";
      token = strtok( (char*)line, delim );
      count++;
   }  
   pclose(fp);
  
   if(count == 0) {
	  printf("snetaid is not running\n");
	  exit(EXIT_FAILURE);  
   }
   else if(count == 1) {
      sbuf_addstr(s, token);
   }
   else {
      printf("Too many instances of snetaid\n");
	  exit(EXIT_FAILURE);
   }
}

void get_daemon_path(struct sbuf *s, const char *pid)
{    
   FILE *fp = NULL;
   char *token = NULL;
   struct sbuf cmd;
  
   sbuf_init(&cmd);
   sbuf_concat(&cmd, 2, "ps -A -o pid,cmd | grep snetaid | grep ", pid);
   fp=popen(cmd.buf, "r");
   if(!fp) {
      printf("Sorry, there was an error opening the pipe\n");
      free(cmd.buf);
      exit(EXIT_FAILURE);
   }
   sbuf_free(&cmd);
   
   char line[MAX_SIZE]={0};
   fgets(line, MAX_SIZE, fp);
   line[strcspn(line, "\n")] = 0;
   const char delim[2] = " ";
   token = strtok((char*)line, delim);
   token = strtok(NULL, token);
   token = strtok(token, delim);
   pclose(fp);

   sbuf_addstr(s, token);
}

// This routine will be used by snetaid; pstat requires root permissions 
// to request information about a foreing process
bool enquery_pstat_binary(const char *cpid, const char *allowed_pstat_binary)
{
   struct sbuf s;
   bool res = false;
   
   sbuf_init(&s);
   get_pstat_binary(&s, cpid);
   
   // Do they match?
   if(!strcmp(s.buf, allowed_pstat_binary)) {
      res = true;
   }
   
   sbuf_free(&s);
 
   return res;
}

void get_pstat_binary(struct sbuf *s, const char *cpid)
{
   char output[MAX_SIZE]; 
   struct sbuf cmd;
   FILE *fp = NULL;
  
   // Get the output of "pstat PID" and inspect the PSTAT_BINARY="value" key pair
   sbuf_init(&cmd);
   sbuf_concat(&cmd, 2, "/usr/bin/pstat ", cpid);
  
   fp = popen(cmd.buf, "r");
   if(!fp) {
      printf("Pipe error\n");
      free(cmd.buf);
      exit(EXIT_FAILURE);
   }
   sbuf_free(&cmd);
   
   while(fgets(output, MAX_SIZE, fp)) {
     const char delim[2] = "=";
     char *token = strtok( (char*)output, delim );
     if(!strcmp(token, "PSTAT_BINARY")) {
        token = strtok(NULL, delim);
        token[strcspn(token, "\n")] = 0;
        sbuf_addstr(s, strdelch(token, '"'));
     }
   }
        
   pclose(fp);
}

int main(void)
{   
   key_t key;
   struct sbuf daemon_pid __cleanbuf__;
   int value, shm_id, *shm_mem;  
   char cwd_buf[ PATH_MAX + 1 ];
   union sigval sv;   
    
   // Get the current directory
   memset( cwd_buf, 0, PATH_MAX + 1 );   
   char* tmp = getcwd( cwd_buf, PATH_MAX );
   if(!tmp) {
      log_error("Current working directory exceeds %u bytes\n", PATH_MAX);
      exit(EXIT_FAILURE);
   }
   
   // Receive the argument (an integer) from the shared memory segment
   key = ftok(cwd_buf, 's');        
   shm_id = shmget(key, sizeof(int), 0666);
   shm_mem = (int*) shmat(shm_id, NULL, 0);
   value = *shm_mem;                
   shmdt(shm_mem);

   // get daemon's pid and send the signal             
   sbuf_init(&daemon_pid); 
   get_daemon_pid(&daemon_pid);
   
   sv.sival_int = value;
   
   if(value != 0) {
      // Send SIGUSR1 with value to snetaid
      if(sigqueue(atoi(daemon_pid.buf), SIGUSR1, sv) != 0) {
        perror("SIGSENT-ERROR:");
      }
   }   
   else {
      // Send SIGUSR2 with value equal to zero
      if(sigqueue(atoi(daemon_pid.buf), SIGUSR2, sv) != 0) {
        perror("SIGSENT-ERROR:");
      }
   }   
   
   _exit(0);
}

