 /*
  * utils.h
  * Copyright (C) Aitor Cuadrado Zubizarreta <aitor_czr@gnuinos.org>
  * 
  * Most of the code of this file is taken from the RTnetlink service routines
  * of the iproute2 project.
  * 
  * https://git.kernel.org/pub/scm/network/iproute2/iproute2.git/
  * 
  * simple-netaid is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
  * Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
  * 
  * simple-netaid is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  * See the GNU General Public License for more details.
  * 
  * You should have received a copy of the GNU General Public License along
  * with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * See the COPYING file.
  */

#ifndef __UTILS_H__
#define __UTILS_H__

#include <resolv.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <stdbool.h>
#include <stddef.h>

#include "libnetlink.h"

extern int preferred_family;
extern int show_stats;

#define SPRINT_BSIZE 64
#define SPRINT_BUF(x)	char x[SPRINT_BSIZE]

typedef struct
{
	__u16 flags;
	__u16 bytelen;
	__s16 bitlen;
	/* These next two fields match rtvia */
	__u16 family;
	__u32 data[8];
} inet_prefix;

extern int cmdlineno;

char* strncpy_t(char*, size_t, const char*, size_t);
char *strdelch(char*, char);
int af_bit_len(int);
int matches(const char*, const char*);
ssize_t getcmdline(char**, size_t*, FILE*);
int makeargs(char*, char**, int);
int get_real_family(int, int);
short file_exists(const char*);

extern int inet_addr_match(const inet_prefix*, const inet_prefix*, int);

#endif /* __UTILS_H__ */
